function results = my_canny(input, sigma, LThres, UThres)
% my_canny:
% - input ... input image (dip_image or matrix)
% - sigma ... width of Gaussian
% - LThres ... lower threshold for hysteresis thresholding
% - UThres ... upper threshold for hysteresis thresholding
% 
% example of use:
% >> I = readim('erika');
% >> dip_image(my_canny(I, 7, 2, 5))

% process the input
input = double(input);
radius = 3*sigma;

% prepare 1D Gaussian
g = exp((-[-radius:radius].^2)/(2*sigma));
g = g/sum(g);

% convolve the input image with two perpendicular 1D Gausian kernels
fx = conv2(input, g, 'same');
fy = conv2(input, g', 'same');

% find the 1st derivative of 1D Gaussian
dg = g(2:end) - g(1:end-1);
gx = dg;  % for x-axis
gy = dg'; % for y-axis

% convolve the smoothed input image with 1st derivative of Gaussian
% (in the perpendicular direction)
fx_dy = conv2(fx, gy, 'same');
fy_dx = conv2(fy, gx, 'same');

% find magnitudes
results = sqrt(fx_dy.^2 + fy_dx.^2);

% non-max suppression
% WRITE YOUR CODE HERE 

% call hysteresis thresholding
% results = threshold(results, 'hysteresis', [LThres, UThres]);

end
